@@ -1,8 +1,45 @@
-#####################################
-##                                 ##
-## additions to the Clayton copula ##
-##                                 ##
-#####################################
+
+
+#' Classes \code{"surClaytonCopula"}, \code{"r90ClaytonCopula"} and
+#' \code{"r270ClaytonCopula"}
+#'
+#' A class representing rotated versions of the Clayton copula family
+#' (survival, 90 and 270 degree rotated).
+#'
+#'
+#' @name surClaytonCopula-class
+#' @aliases surClaytonCopula-class dduCopula,matrix,surClaytonCopula-method
+#' dduCopula,numeric,surClaytonCopula-method
+#' ddvCopula,matrix,surClaytonCopula-method
+#' ddvCopula,numeric,surClaytonCopula-method r90ClaytonCopula-class
+#' dduCopula,matrix,r90ClaytonCopula-method
+#' dduCopula,numeric,r90ClaytonCopula-method
+#' ddvCopula,matrix,r90ClaytonCopula-method
+#' ddvCopula,numeric,r90ClaytonCopula-method r270ClaytonCopula-class
+#' dduCopula,matrix,r270ClaytonCopula-method
+#' dduCopula,numeric,r270ClaytonCopula-method
+#' ddvCopula,matrix,r270ClaytonCopula-method
+#' ddvCopula,numeric,r270ClaytonCopula-method
+#' @docType class
+#' @section Objects from the Class: Objects can be created by calls of the form
+#' \code{new("surClaytonCopula", ...)}, \code{new("r90ClaytonCopula", ...)} and
+#' \code{new("r270ClaytonCopula", ...)} or by the function
+#' \code{\link{surClaytonCopula}}, \code{\link{r90ClaytonCopula}} and
+#' \code{\link{r270ClaytonCopula}} respectively.
+#' @author Benedikt Graeler
+#' @seealso \code{\link{VineCopula-package}}
+#' @keywords classes
+#' @examples
+#'
+#' library(copula)
+#'
+#' persp(surClaytonCopula(.5),dCopula,zlim=c(0,10))
+#' persp(r90ClaytonCopula(-.5),dCopula,zlim=c(0,10))
+#' persp(r270ClaytonCopula(-.5),dCopula,zlim=c(0,10))
+#'
+NULL
+
+
 
 #############################
 ## Clayton survival copula ##
@@ -30,21 +67,44 @@ setClass("surClaytonCopula",
 )
 
 # constructor
+
+
+#' Survival and Rotated Clayton Copulas
+#'
+#' These are wrappers to functions from \code{\link{VineCopula-package}}
+#'
+#'
+#' @aliases surClaytonCopula r90ClaytonCopula r270ClaytonCopula
+#' @param param A single parameter defining the Copula.
+#' @return An object of class \code{\linkS4class{surClaytonCopula}},
+#' \code{\linkS4class{r90ClaytonCopula}} or
+#' \code{\linkS4class{r270ClaytonCopula}} respectively.
+#' @author Benedikt Graeler
+#' @keywords copula
+#' @examples
+#'
+#' library(copula)
+#'
+#' persp(surClaytonCopula(1.5), dCopula, zlim = c(0,10))
+#' persp(r90ClaytonCopula(-1.5), dCopula, zlim = c(0,10))
+#' persp(r270ClaytonCopula(-1.5), dCopula, zlim = c(0,10))
+#'
+#' @export surClaytonCopula
 surClaytonCopula <- function (param=1) {
   new("surClaytonCopula", dimension = as.integer(2), parameters = param, param.names = c("theta"),
-      param.lowbnd = 0, param.upbnd = Inf, family=13, 
+      param.lowbnd = 0, param.upbnd = Inf, family=13,
       fullname = "Survival Clayton copula family. Number 13 in VineCopula.")
 }
 
 ## density ##
-setMethod("dCopula", signature("numeric","surClaytonCopula"), 
+setMethod("dCopula", signature("numeric","surClaytonCopula"),
           function(u, copula, log) {
             linkVineCop.PDF(matrix(u,ncol=copula@dimension),copula, log)
           })
 setMethod("dCopula", signature("matrix","surClaytonCopula"), linkVineCop.PDF)
 
 ## jcdf ##
-setMethod("pCopula", signature("numeric","surClaytonCopula"), 
+setMethod("pCopula", signature("numeric","surClaytonCopula"),
           function(u, copula) {
             linkVineCop.surCDF(matrix(u,ncol=copula@dimension),copula)
           })
@@ -52,14 +112,14 @@ setMethod("pCopula", signature("matrix","surClaytonCopula"), linkVineCop.surCDF)
 
 ## partial derivatives ##
 # ddu
-setMethod("dduCopula", signature("numeric","surClaytonCopula"), 
+setMethod("dduCopula", signature("numeric","surClaytonCopula"),
           function(u, copula, log) {
             linkVineCop.ddu(matrix(u,ncol=copula@dimension),copula)
           })
 setMethod("dduCopula", signature("matrix","surClaytonCopula"), linkVineCop.ddu)
 
 # ddv
-setMethod("ddvCopula", signature("numeric","surClaytonCopula"), 
+setMethod("ddvCopula", signature("numeric","surClaytonCopula"),
           function(u, copula, log) {
             linkVineCop.ddv(matrix(u,ncol=copula@dimension),copula)
           })
@@ -69,9 +129,9 @@ setMethod("ddvCopula", signature("matrix","surClaytonCopula"), linkVineCop.ddv)
 setMethod("rCopula", signature("numeric","surClaytonCopula"), linkVineCop.r)
 
 ## Kendalls tau to parameter conversion
-setMethod("iTau", signature("surClaytonCopula"), 
+setMethod("iTau", signature("surClaytonCopula"),
           function(copula, tau) {
-            if(tau <= 0) 
+            if(tau <= 0)
               return(NA)
             linkVineCop.iTau(copula, max(1e-6,abs(tau)))
           })
@@ -107,19 +167,19 @@ setClass("r90ClaytonCopula",
 # constructor
 r90ClaytonCopula <- function (param=-1) {
   new("r90ClaytonCopula", dimension = as.integer(2), parameters = param, param.names = c("theta", "delta"),
-      param.lowbnd = -Inf, param.upbnd = 0, family=23, 
+      param.lowbnd = -Inf, param.upbnd = 0, family=23,
       fullname = "90 deg rotated Clayton copula family. Number 23 in VineCopula.")
 }
 
 ## density ##
-setMethod("dCopula", signature("numeric","r90ClaytonCopula"), 
+setMethod("dCopula", signature("numeric","r90ClaytonCopula"),
           function(u, copula, log) {
             linkVineCop.PDF(matrix(u,ncol=copula@dimension),copula, log)
           })
 setMethod("dCopula", signature("matrix","r90ClaytonCopula"), linkVineCop.PDF)
 
 ## jcdf ##
-setMethod("pCopula", signature("numeric","r90ClaytonCopula"), 
+setMethod("pCopula", signature("numeric","r90ClaytonCopula"),
           function(u, copula) {
             linkVineCop.r90CDF(matrix(u,ncol=copula@dimension),copula)
           })
@@ -127,14 +187,14 @@ setMethod("pCopula", signature("matrix","r90ClaytonCopula"), linkVineCop.r90CDF)
 
 ## partial derivatives ##
 # ddu
-setMethod("dduCopula", signature("numeric","r90ClaytonCopula"), 
+setMethod("dduCopula", signature("numeric","r90ClaytonCopula"),
           function(u, copula) {
             linkVineCop.ddu(matrix(u,ncol=copula@dimension),copula)
           })
 setMethod("dduCopula", signature("matrix","r90ClaytonCopula"), linkVineCop.ddu)
 
 ## ddv
-setMethod("ddvCopula", signature("numeric","r90ClaytonCopula"), 
+setMethod("ddvCopula", signature("numeric","r90ClaytonCopula"),
           function(u, copula) {
             linkVineCop.ddv(matrix(u,ncol=copula@dimension),copula)
           })
@@ -146,7 +206,7 @@ setMethod("rCopula", signature("numeric","r90ClaytonCopula"), linkVineCop.r)
 ## Kendalls tau to parameter conversion
 setMethod("iTau", signature("r90ClaytonCopula"),
           function(copula, tau) {
-            if(tau >= 0) 
+            if(tau >= 0)
               return(NA)
             linkVineCop.iTau(copula, min(-1e-6,-abs(tau)))
           })
@@ -166,20 +226,20 @@ setClass("r270ClaytonCopula",
 
 # constructor
 r270ClaytonCopula <- function (param=-1) {
-  new("r270ClaytonCopula", dimension = as.integer(2), parameters = param, param.names = c("theta", "delta"), 
-      param.lowbnd = -Inf, param.upbnd = 0, family=33, 
+  new("r270ClaytonCopula", dimension = as.integer(2), parameters = param, param.names = c("theta", "delta"),
+      param.lowbnd = -Inf, param.upbnd = 0, family=33,
       fullname = "270 deg rotated Clayton copula family. Number 33 in VineCopula.")
 }
 
 ## density ##
-setMethod("dCopula", signature("numeric","r270ClaytonCopula"), 
+setMethod("dCopula", signature("numeric","r270ClaytonCopula"),
           function(u, copula, log) {
             linkVineCop.PDF(matrix(u,ncol=copula@dimension),copula, log)
           })
 setMethod("dCopula", signature("matrix","r270ClaytonCopula"), linkVineCop.PDF)
 
 ## jcdf ##
-setMethod("pCopula", signature("numeric","r270ClaytonCopula"), 
+setMethod("pCopula", signature("numeric","r270ClaytonCopula"),
           function(u, copula) {
             linkVineCop.r270CDF(matrix(u,ncol=copula@dimension),copula)
           })
@@ -187,14 +247,14 @@ setMethod("pCopula", signature("matrix","r270ClaytonCopula"), linkVineCop.r270CD
 
 ## partial derivatives ##
 # ddu
-setMethod("dduCopula", signature("numeric","r270ClaytonCopula"), 
+setMethod("dduCopula", signature("numeric","r270ClaytonCopula"),
           function(u, copula) {
             linkVineCop.ddu(matrix(u,ncol=copula@dimension),copula)
           })
 setMethod("dduCopula", signature("matrix","r270ClaytonCopula"), linkVineCop.ddu)
 
 ## ddv
-setMethod("ddvCopula", signature("numeric","r270ClaytonCopula"), 
+setMethod("ddvCopula", signature("numeric","r270ClaytonCopula"),
           function(u, copula) {
             linkVineCop.ddv(matrix(u,ncol=copula@dimension),copula)
           })
@@ -204,9 +264,9 @@ setMethod("ddvCopula", signature("matrix","r270ClaytonCopula"), linkVineCop.ddv)
 setMethod("rCopula", signature("numeric","r270ClaytonCopula"), linkVineCop.r)
 
 ## Kendalls tau to parameter conversion
-setMethod("iTau", signature("r270ClaytonCopula"), 
+setMethod("iTau", signature("r270ClaytonCopula"),
           function(copula, tau) {
-            if(tau >= 0) 
+            if(tau >= 0)
               return(NA)
             linkVineCop.iTau(copula, min(-1e-6,-abs(tau)))
           })
@@ -215,11 +275,48 @@ setMethod("tau",signature("r270ClaytonCopula"),linkVineCop.tau)
 
 setMethod("tailIndex",signature("r270ClaytonCopula"),linkVineCop.tailIndex)
 
-####################################
-##                                ##
-## additions to the Gumbel copula ##
-##                                ##
-####################################
+
+
+
+#' Classes \code{"surGumbelCopula"}, \code{"r90GumbelCopula"} and
+#' \code{"r270GumbelCopula"}
+#'
+#' A class representing rotated versions of the Gumbel copula family (survival,
+#' 90 and 270 degree rotated).
+#'
+#'
+#' @name surGumbelCopula-class
+#' @aliases surGumbelCopula-class dduCopula,matrix,surGumbelCopula-method
+#' dduCopula,numeric,surGumbelCopula-method
+#' ddvCopula,matrix,surGumbelCopula-method
+#' ddvCopula,numeric,surGumbelCopula-method r90GumbelCopula-class
+#' dduCopula,matrix,r90GumbelCopula-method
+#' dduCopula,numeric,r90GumbelCopula-method
+#' ddvCopula,matrix,r90GumbelCopula-method
+#' ddvCopula,numeric,r90GumbelCopula-method r270GumbelCopula-class
+#' dduCopula,matrix,r270GumbelCopula-method
+#' dduCopula,numeric,r270GumbelCopula-method
+#' ddvCopula,matrix,r270GumbelCopula-method
+#' ddvCopula,numeric,r270GumbelCopula-method
+#' @docType class
+#' @section Objects from the Class: Objects can be created by calls of the form
+#' \code{new("surGumbelCopula", ...)}, \code{new("r90GumbelCopula", ...)} and
+#' \code{new("r270GumbelCopula", ...)} or by the function
+#' \code{\link{surGumbelCopula}}, \code{\link{r90GumbelCopula}} and
+#' \code{\link{r270GumbelCopula}} respectively.
+#' @author Benedikt Graeler
+#' @seealso \code{\link{VineCopula-package}}
+#' @keywords classes
+#' @examples
+#'
+#' library(copula)
+#'
+#' persp(surGumbelCopula(1.5),dCopula,zlim=c(0,10))
+#' persp(r90GumbelCopula(-1.5),dCopula,zlim=c(0,10))
+#' persp(r270GumbelCopula(-1.5),dCopula,zlim=c(0,10))
+#'
+NULL
+
 
 validGumbelCopula = function(object) {
   if (object@dimension != 2)
@@ -247,21 +344,44 @@ setClass("surGumbelCopula",
          )
 
 # constructor
+
+
+#' Survival and Rotated Gumbel Copulas
+#'
+#' These are wrappers to functions from \code{\link{VineCopula-package}}
+#'
+#'
+#' @aliases surGumbelCopula r90GumbelCopula r270GumbelCopula
+#' @param param A single parameter defining the Copula.
+#' @return An object of class \code{\linkS4class{surGumbelCopula}},
+#' \code{\linkS4class{r90GumbelCopula}} or
+#' \code{\linkS4class{r270GumbelCopula}} respectively.
+#' @author Benedikt Graeler
+#' @keywords copula
+#' @examples
+#'
+#' library(copula)
+#'
+#' persp(surGumbelCopula(1.5), dCopula, zlim = c(0,10))
+#' persp(r90GumbelCopula(-1.5), dCopula, zlim = c(0,10))
+#' persp(r270GumbelCopula(-1.5), dCopula, zlim = c(0,10))
+#'
+#' @export surGumbelCopula
 surGumbelCopula <- function (param=1) {
   new("surGumbelCopula", dimension = as.integer(2), parameters = param, param.names = c("theta"),
-      param.lowbnd = 1, param.upbnd = Inf, family=14, 
+      param.lowbnd = 1, param.upbnd = Inf, family=14,
       fullname = "Survival Gumbel copula family. Number 14 in VineCopula.")
 }
 
 ## density ##
-setMethod("dCopula", signature("numeric","surGumbelCopula"), 
+setMethod("dCopula", signature("numeric","surGumbelCopula"),
           function(u, copula, log) {
             linkVineCop.PDF(matrix(u,ncol=copula@dimension),copula, log)
           })
 setMethod("dCopula", signature("matrix","surGumbelCopula"), linkVineCop.PDF)
 
 ## jcdf ##
-setMethod("pCopula", signature("numeric","surGumbelCopula"), 
+setMethod("pCopula", signature("numeric","surGumbelCopula"),
           function(u, copula) {
             linkVineCop.surCDF(matrix(u,ncol=copula@dimension),copula)
           })
@@ -269,14 +389,14 @@ setMethod("pCopula", signature("matrix","surGumbelCopula"), linkVineCop.surCDF)
 
 ## partial derivatives ##
 # ddu
-setMethod("dduCopula", signature("numeric","surGumbelCopula"), 
+setMethod("dduCopula", signature("numeric","surGumbelCopula"),
           function(u, copula) {
             linkVineCop.ddu(matrix(u,ncol=copula@dimension),copula)
           })
 setMethod("dduCopula", signature("matrix","surGumbelCopula"), linkVineCop.ddu)
 
 # ddv
-setMethod("ddvCopula", signature("numeric","surGumbelCopula"), 
+setMethod("ddvCopula", signature("numeric","surGumbelCopula"),
           function(u, copula) {
             linkVineCop.ddv(matrix(u,ncol=copula@dimension),copula)
           })
@@ -286,9 +406,9 @@ setMethod("ddvCopula", signature("matrix","surGumbelCopula"), linkVineCop.ddv)
 setMethod("rCopula", signature("numeric","surGumbelCopula"), linkVineCop.r)
 
 ## Kendalls tau to parameter conversion
-setMethod("iTau", signature("surGumbelCopula"), 
+setMethod("iTau", signature("surGumbelCopula"),
           function(copula, tau) {
-            if(tau < 0) 
+            if(tau < 0)
               return(NA)
             linkVineCop.iTau(copula, max(0,abs(tau)))
           })
@@ -325,19 +445,19 @@ setClass("r90GumbelCopula",
 # constructor
 r90GumbelCopula <- function (param=-1) {
   new("r90GumbelCopula", dimension = as.integer(2), parameters = param, param.names = c("theta", "delta"),
-      param.lowbnd = -Inf, param.upbnd = -1, family=24, 
+      param.lowbnd = -Inf, param.upbnd = -1, family=24,
       fullname = "90 deg rotated Gumbel copula family. Number 24 in VineCopula.")
 }
 
 ## density ##
-setMethod("dCopula", signature("numeric","r90GumbelCopula"), 
+setMethod("dCopula", signature("numeric","r90GumbelCopula"),
           function(u, copula, log) {
             linkVineCop.PDF(matrix(u,ncol=copula@dimension),copula, log)
           })
 setMethod("dCopula", signature("matrix","r90GumbelCopula"), linkVineCop.PDF)
 
 ## jcdf ##
-setMethod("pCopula", signature("numeric","r90GumbelCopula"), 
+setMethod("pCopula", signature("numeric","r90GumbelCopula"),
           function(u, copula) {
             linkVineCop.r90CDF(matrix(u,ncol=copula@dimension),copula)
           })
@@ -345,14 +465,14 @@ setMethod("pCopula", signature("matrix","r90GumbelCopula"), linkVineCop.r90CDF)
 
 ## partial derivatives ##
 # ddu
-setMethod("dduCopula", signature("numeric","r90GumbelCopula"), 
+setMethod("dduCopula", signature("numeric","r90GumbelCopula"),
           function(u, copula) {
             linkVineCop.ddu(matrix(u,ncol=copula@dimension),copula)
           })
 setMethod("dduCopula", signature("matrix","r90GumbelCopula"), linkVineCop.ddu)
 
 ## ddv
-setMethod("ddvCopula", signature("numeric","r90GumbelCopula"), 
+setMethod("ddvCopula", signature("numeric","r90GumbelCopula"),
           function(u, copula) {
             linkVineCop.ddv(matrix(u,ncol=copula@dimension),copula)
           })
@@ -364,7 +484,7 @@ setMethod("rCopula", signature("numeric","r90GumbelCopula"), linkVineCop.r)
 ## Kendalls tau to parameter conversion
 setMethod("iTau", signature("r90GumbelCopula"),
           function(copula, tau) {
-            if(tau > 0) 
+            if(tau > 0)
               return(NA)
             linkVineCop.iTau(copula, min(0,-abs(tau)))
           })
@@ -385,20 +505,20 @@ setClass("r270GumbelCopula",
 
 # constructor
 r270GumbelCopula <- function (param=-1) {
-  new("r270GumbelCopula", dimension = as.integer(2), parameters = param, param.names = c("theta", "delta"), 
-      param.lowbnd = -Inf, param.upbnd = -1, family=34, 
+  new("r270GumbelCopula", dimension = as.integer(2), parameters = param, param.names = c("theta", "delta"),
+      param.lowbnd = -Inf, param.upbnd = -1, family=34,
       fullname = "270 deg rotated Gumbel copula family. Number 34 in VineCopula.")
 }
 
 ## density ##
-setMethod("dCopula", signature("numeric","r270GumbelCopula"), 
+setMethod("dCopula", signature("numeric","r270GumbelCopula"),
           function(u, copula, log) {
             linkVineCop.PDF(matrix(u,ncol=copula@dimension),copula, log)
           })
 setMethod("dCopula", signature("matrix","r270GumbelCopula"), linkVineCop.PDF)
 
 ## jcdf ##
-setMethod("pCopula", signature("numeric","r270GumbelCopula"), 
+setMethod("pCopula", signature("numeric","r270GumbelCopula"),
           function(u, copula) {
             linkVineCop.r270CDF(matrix(u,ncol=copula@dimension),copula)
           })
@@ -406,14 +526,14 @@ setMethod("pCopula", signature("matrix","r270GumbelCopula"), linkVineCop.r270CDF
 
 ## partial derivatives ##
 # ddu
-setMethod("dduCopula", signature("numeric","r270GumbelCopula"), 
+setMethod("dduCopula", signature("numeric","r270GumbelCopula"),
           function(u, copula) {
             linkVineCop.ddu(matrix(u,ncol=copula@dimension),copula)
           })
 setMethod("dduCopula", signature("matrix","r270GumbelCopula"), linkVineCop.ddu)
 
 ## ddv
-setMethod("ddvCopula", signature("numeric","r270GumbelCopula"), 
+setMethod("ddvCopula", signature("numeric","r270GumbelCopula"),
           function(u, copula) {
             linkVineCop.ddv(matrix(u,ncol=copula@dimension),copula)
           })
@@ -423,7 +543,7 @@ setMethod("ddvCopula", signature("matrix","r270GumbelCopula"), linkVineCop.ddv)
 setMethod("rCopula", signature("numeric","r270GumbelCopula"), linkVineCop.r)
 
 ## Kendalls tau to parameter conversion
-setMethod("iTau", signature("r270GumbelCopula"), 
+setMethod("iTau", signature("r270GumbelCopula"),
           function(copula, tau) {
             if(tau >= 0)
               return(NA)
