@@ -29,6 +29,8 @@
 #' @param level numeric; significance level of the independence test (default:
 #' \code{level = 0.05}).
 #' @param trunclevel integer; level of truncation.
+#' @param se Logical; whether standard errors are estimated (default: \code{se
+#' = FALSE}).
 #' @param rotations logical; if \code{TRUE}, all rotations of the families in
 #' \code{familyset} are included.
 #' @param cores integer; if \code{cores > 1}, estimation will be parallized
@@ -117,7 +119,7 @@
 #' contour(RVM1)  # contour plots of all pair-copulas
 #'
 RVineCopSelect <- function(data, familyset = NA, Matrix, selectioncrit = "AIC", indeptest = FALSE,
-                           level = 0.05, trunclevel = NA, rotations = TRUE, cores = 1) {
+                           level = 0.05, trunclevel = NA, se = FALSE, rotations = TRUE, cores = 1) {
     ## preprocessing of arguments
     args <- preproc(c(as.list(environment()), call = match.call()),
                     check_data,
@@ -229,7 +231,7 @@ RVineCopSelect <- function(data, familyset = NA, Matrix, selectioncrit = "AIC",
                                                          level,
                                                          weights = NA,
                                                          rotations,
-                                                         se = TRUE))
+                                                         se = se))
                     warn <- NULL
                 }
 
@@ -270,7 +272,8 @@ RVineCopSelect <- function(data, familyset = NA, Matrix, selectioncrit = "AIC",
             Types[k, i]   <- res.k[[i]]$cfit$family
             Params[k, i]  <- res.k[[i]]$cfit$par
             Params2[k, i] <- res.k[[i]]$cfit$par2
-            Ses[k, i]     <- res.k[[i]]$cfit$se
+            tmpse         <- res.k[[i]]$cfit$se
+            Ses[k, i]     <- ifelse(is.null(tmpse), NA, tmpse)
             tmpse2        <- res.k[[i]]$cfit$se2
             Se2s[k, i]    <- ifelse(is.null(tmpse2), NA, tmpse2)
             emptaus[k, i] <- res.k[[i]]$cfit$emptau
@@ -293,8 +296,10 @@ RVineCopSelect <- function(data, familyset = NA, Matrix, selectioncrit = "AIC",
                         par = Params,
                         par2 = Params2,
                         names = varnames)
-    .RVM$se <- Ses
-    .RVM$se2 <- Se2s
+    if (se == TRUE) {
+        .RVM$se <- Ses
+        .RVM$se2 <- Se2s
+    }
     .RVM$nobs <- N
     revo <- sapply(1:d, function(i) which(o[length(o):1] == i))
     like <- suppressWarnings(RVineLogLik(data[, revo], .RVM))
