@@ -58,7 +58,7 @@ void CvMtest(double* cdf, int* n, double* out)
 {
   int i;
   double sum1=0.0,sum2=0.0;
-  for(i=0;i<*n;i++) 
+  for(i=0;i<*n;i++)
   {
     sum1 += pow(cdf[i],2.0);
     sum2 += cdf[i]*(2.0*((double)i+1.0)+1.0);
@@ -98,7 +98,7 @@ void White(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect
 {
 	int i=0, dd=0, tt=0, k=1, j=0, kk=0, t=0, mm=0, dd2=0;
 	double *Dprime, *hess, *subhess, *der, *subder, *dat, *hess_red, *der_red;
-	
+
 	for(i=0; i<(*d*(*d));i++)
 	{
 		if(family[i]!=0) dd++;
@@ -106,7 +106,7 @@ void White(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect
 	}
 	mm=(dd+tt)*(dd+tt+1)/2;
 	dd2=*d*(*d-1)/2;
-	
+
 	//Allocate memory
 	Dprime = malloc((dd+tt)*(dd+tt+1)/2*sizeof(double));
 	hess = malloc((dd2+tt)*(dd2+tt)*sizeof(double));
@@ -116,14 +116,14 @@ void White(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect
 	hess_red = malloc((dd+tt)*(dd+tt)*sizeof(double));
 	der_red = malloc((dd+tt)*(dd+tt)*sizeof(double));
 	dat = malloc(*d*sizeof(double));
-	
+
 	// initialisieren
 	for(i=0;i<mm;i++)
 	{
 		Dprime[i]=0;
 	}
-	
-	
+
+
 	for(t=0;t<*T;t++)
 	{
 		for(i=0; i<*d;i++)
@@ -138,7 +138,7 @@ void White(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect
 			subder[i]=0;
 		}
 		hesse(&k, d, family, maxmat, matrix, condirect, conindirect, par, par2, dat, hess, subhess, der, subder);
-		
+
 		// independence aus der Hesse herausnehmen
 		kk=0;
 		for(i=0;i<dd2+tt;i++)
@@ -153,7 +153,7 @@ void White(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect
 				}
 			}
 		}
-		
+
 		kk=0;
 		for(i=0;i<(dd+tt);i++)
 		{
@@ -164,7 +164,7 @@ void White(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect
 				kk++;
 			}
 		}
-		
+
 		for(i=0;i<mm;i++)
 		{
 			for(j=0;j<mm;j++)
@@ -172,12 +172,12 @@ void White(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect
 				V[(i+1)+mm*j-1]+=(Dprime[i]*Dprime[j]/(double)(*T));
 			}
 		}
-	} 
-	
+	}
+
 	// Nicht fertig, da hier das Problem D%*%solve(V)%*%t(D) zu loesen ist
-	
+
 	// Free memory
-	free(Dprime);	
+	free(Dprime);
 	free(hess);
 	free(subhess);
 	free(der);
@@ -195,10 +195,10 @@ void White(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect
 
 
 ///////////////////////////////////////////////////////////////////////////////
-// Function to compute probability of observing rank i, given rank 1,...,i-1 
+// Function to compute probability of observing rank i, given rank 1,...,i-1
 // Help function by Daniel Berg
-// Input: q (vector to be transformed, must be sorted ascending), 
-// d(length of vector q), out(output) 
+// Input: q (vector to be transformed, must be sorted ascending),
+// d(length of vector q), out(output)
 ///////////////////////////////////////////////////////////////////////////////
 
 void ZStar(double* q, int* d, double* out)
@@ -221,10 +221,10 @@ void ZStar(double* q, int* d, double* out)
 // Function to compare two numbers
 // Help function by Daniel Berg
 ///////////////////////////////////////////////////////////////////////////////
-int comp_nums(double *num1, double *num2)
+int comp_nums(const void *num1, const void *num2)
 {
-  if (*num1 <  *num2) return -1;
-  else if (*num1 == *num2) return  0;
+  if (*(double*)num1 <  *(double*)num2) return -1;
+  else if (*(double*)num1 == *(double*)num2) return  0;
   else return  1;
 }
 
@@ -243,8 +243,9 @@ int comp_nums(double *num1, double *num2)
 // out			sum of transformed data (PIT) (one step for the Breymann, Berg and Berg2 GOF)
 //////////////////////////////////////////
 
-void Bj(int *T, int* d, int* family, int* maxmat, int* matrix, int* condirect, int* conindirect, double* par, double* par2, double* data, 
-		 double* out, double* vv, double* vv2, int* calcupdate, int* method, int *alpha)
+void Bj(int *T, int* d, int* family, int* maxmat, int* matrix, int* condirect,
+        int* conindirect, double* par, double* par2, double* data, double* out,
+        double* vv, double* vv2, int* calcupdate, int* method, int *alpha)
 {
 	int i=0, t=0, ii=0, j=0;
 	double *udata, **tmp, **u;
@@ -253,7 +254,7 @@ void Bj(int *T, int* d, int* family, int* maxmat, int* matrix, int* condirect, i
 	u = create_matrix(*T,*d);
 
 	RvinePIT(T, d, family, maxmat, matrix, condirect, conindirect, par, par2, data, udata, vv, vv2, calcupdate);
-	
+
 	ii=0;
 	for(j=0;j<*T;j++)
 	{
@@ -264,7 +265,7 @@ void Bj(int *T, int* d, int* family, int* maxmat, int* matrix, int* condirect, i
 				u[j][i] = udata[ii];
 				ii += 1;
 			}
-			qsort(u[j],*d,sizeof(double),(void *)comp_nums);
+			qsort(u[j], *d, sizeof(double), comp_nums);
 			ZStar(u[j],d,tmp[j]);		//Transformation von Berg and Bakken (2007); ordered PIT
 		}
 		else	// Im Fall von Breymann ist es besser keine Transformation zu machen
@@ -276,7 +277,7 @@ void Bj(int *T, int* d, int* family, int* maxmat, int* matrix, int* condirect, i
 			}
 		}
 	}
-	
+
 	for(t=0;t<*T;t++)
 	{
 		for(i=0;i<*d;i++)
@@ -287,11 +288,11 @@ void Bj(int *T, int* d, int* family, int* maxmat, int* matrix, int* condirect, i
 				tmp[t][i]=fabs(tmp[t][i]-0.5);
 			else if(*method==3)  // Berg2: (u-0.5)^alpha
 				tmp[t][i]=pow(tmp[t][i]-0.5,*alpha);
-			
+
 			out[t]+=tmp[t][i];
 		}
 	}
-	
+
 	free(udata);
 	free_matrix(tmp,*T);
 	free_matrix(u,*T);
@@ -316,20 +317,20 @@ void SimulateBj(double* S, int *T, int* d, int* B, int* method, int *alpha, doub
 	double *tmp, Sb=0, *ustar;
 	tmp = malloc(*d*sizeof(double));
 	ustar = malloc(*d*sizeof(double));
-	
+
 	GetRNGstate();		// random number generator
-	
+
 	for(t=0;t<*T;t++)
 	{
 		p[t]=0;
 	}
-	
+
 	for(m=0;m<*B;m++)
 	{
 		for(i=1;i<=*d;i++) { tmp[i] = runif(0,1);}
-		qsort(tmp,*d,sizeof(double),(void *)comp_nums);
+		qsort(tmp, *d, sizeof(double), comp_nums);
 		ZStar(tmp,d,ustar);		//Transformation von Berg and Bakken (2007)
-	
+
 		for(i=0;i<*d;i++)
 		{
 			if(*method==1)
@@ -338,7 +339,7 @@ void SimulateBj(double* S, int *T, int* d, int* B, int* method, int *alpha, doub
 				tmp[i]=fabs(ustar[i]-0.5);
 			else if(*method==3)
 				tmp[i]=pow(ustar[i]-0.5,*alpha);
-			
+
 			Sb+=tmp[i];
 		}
 		for(t=0;t<*T;t++)
@@ -347,13 +348,13 @@ void SimulateBj(double* S, int *T, int* d, int* B, int* method, int *alpha, doub
 		}
 		Sb=0;
 	}
-	for(t=0;t<*T;t++) 
+	for(t=0;t<*T;t++)
 	{
 		if(p[t] == 0) p[t] = 1.0/(1.0+(double)*B);
 	}
-	
+
 	PutRNGstate();
-	
+
 	free(tmp);
 	free(ustar);
 }
@@ -374,7 +375,7 @@ void SimulateBj(double* S, int *T, int* d, int* B, int* method, int *alpha, doub
 // statistic		test statistic
 /////////////////////////////////
 
-void gofPIT_AD(int *T, int* d, int* family, int* maxmat, int* matrix, int* condirect, int* conindirect, double* par, double* par2, double* data, 
+void gofPIT_AD(int *T, int* d, int* family, int* maxmat, int* matrix, int* condirect, int* conindirect, double* par, double* par2, double* data,
 		 double* statistic, double* vv, double* vv2, int* calcupdate, int* method, int *alpha, int* B, int *statisticName)
 {
 	int t=0;
@@ -382,16 +383,16 @@ void gofPIT_AD(int *T, int* d, int* family, int* maxmat, int* matrix, int* condi
 	S = malloc(*T*sizeof(double));
 	helpvar = malloc(*T*sizeof(double));
 	Bhat = malloc(*T*sizeof(double));
-	
+
 	for(t=0;t<*T;t++)
 	{
 		S[t]=0;
 		helpvar[t]=0;
 		Bhat[t]=0;
 	}
-	
+
 	Bj(T, d, family, maxmat, matrix, condirect, conindirect, par, par2, data, S, vv, vv2, calcupdate, method, alpha);
-	
+
 	// Statistic berechnen
 	if(*B==0)  // if an asymptotic based test statistic should be returned
 	{
@@ -404,7 +405,7 @@ void gofPIT_AD(int *T, int* d, int* family, int* maxmat, int* matrix, int* condi
 		}
 		else
 			CumDist(S, T, T, Bhat);  // for the other two we need the empirical distribution function
-		
+
 		if(*statisticName==1)		//Anderson-Darling
 			ADtest(Bhat, T, statistic);
 		else if(*statisticName==2)	//Kolmogorov-Smirnov
@@ -413,7 +414,7 @@ void gofPIT_AD(int *T, int* d, int* family, int* maxmat, int* matrix, int* condi
 		{
 			CvMtest(Bhat, T, statistic);
 		}
-			
+
 	}
 	else		//bootstrap
 	{
@@ -428,7 +429,7 @@ void gofPIT_AD(int *T, int* d, int* family, int* maxmat, int* matrix, int* condi
 			CvMtest(Bhat, T, statistic);
 		}
 	}
-	
+
 	free(S);
 	free(helpvar);
 	free(Bhat);
@@ -449,7 +450,7 @@ void gofPIT_AD_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int
 	int i=0, j=0, m=0, t=0, *f, B2=1000;
 	double *bdata, bstat=0;
 	double *bvv, *bvv2;
-	
+
 	f = malloc(*T*sizeof(int));
 	bdata = malloc(*d*(*T)*sizeof(double));
 	bvv = malloc(*d*(*d)*(*T)*sizeof(double));
@@ -472,13 +473,13 @@ void gofPIT_AD_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int
 			}
 		}
 		bstat=0;
-		gofPIT_AD(T, d, family, maxmat, matrix, condirect, conindirect, par, par2, bdata, 
+		gofPIT_AD(T, d, family, maxmat, matrix, condirect, conindirect, par, par2, bdata,
 				&bstat, bvv, bvv2, calcupdate, method, alpha, &B2, statisticName);
-		
+
 		if(bstat>=*statistic)
 			*pvalue+=1.0/(*B);
 	}
-	
+
 	free(f);
 	free(bdata);
 	free(bvv);
@@ -490,7 +491,7 @@ void gofPIT_AD_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int
 /* Equal probability sampling; with-replacement case */
 // Input:
 // k		how many samples
-// n		max value of sample 
+// n		max value of sample
 //
 // output:
 // y		vector of length k returning the samples
@@ -499,7 +500,7 @@ void gofPIT_AD_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int
 void MySample(int *k, int *n, int *y)
 {
     int i;
-	
+
 	GetRNGstate();
     for (i = 0; i < *k; i++)
 	{
@@ -529,7 +530,7 @@ void gofECP(int* T, int* d, int* family, int* maxmat, int* matrix, int* conindir
 	znull = malloc(*d*1000*sizeof(double));
 	Chat1 = malloc(*T*sizeof(double));
 	Chat2 = malloc(*T*sizeof(double));
-	
+
 	for(t=0;t<T2;t++)
 	{
 		for(i=0;i<*d;i++)
@@ -539,11 +540,11 @@ void gofECP(int* T, int* d, int* family, int* maxmat, int* matrix, int* conindir
 	}
 
 	SimulateRVine(&T2, d, family, maxmat, matrix, conindirect, par, par2, znull, &U, &takeU);
-	
-	
+
+
 	ChatZj(data, data, T, d, T, Chat1);		// empirical copula distribution
 	ChatZj(znull, data, T, d, &T2, Chat2);
-	
+
 	*statistic=0;
 	if(*statisticName==3)	//Cramer-von Mises test statistic
 	{
@@ -560,7 +561,7 @@ void gofECP(int* T, int* d, int* family, int* maxmat, int* matrix, int* conindir
 		}
 		*statistic=*statistic*sqrt(*T);
 	}
-	
+
 	free(znull);
 	free(Chat1);
 	free(Chat2);
@@ -575,7 +576,7 @@ void gofECP_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int* c
 {
 	int i=0, m=0, t=0, *f;
 	double *bdata, bstat=0;
-	
+
 	f = malloc(*T*sizeof(int));
 	bdata = malloc(*d*(*T)*sizeof(double));
 
@@ -595,7 +596,7 @@ void gofECP_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int* c
 		if(bstat>=*statistic)
 			*pvalue+=1.0/(*B);
 	}
-	
+
 	free(f);
 	free(bdata);
 }
@@ -615,7 +616,7 @@ void ChatZj(double* data, double* u, int* n, int* d, int* m, double* Chat)
 	int i,j,k;
 	double *helpvar;
 	helpvar=malloc(*m*sizeof(double));
-	
+
 	for(j=0;j<*n;j++)
 	{
 		Chat[j]=0;
@@ -632,7 +633,7 @@ void ChatZj(double* data, double* u, int* n, int* d, int* m, double* Chat)
 		}
 		Chat[j]/=(*m+1);
 	}
-	
+
 	free(helpvar);
 }
 
@@ -641,10 +642,10 @@ void ChatZj(double* data, double* u, int* n, int* d, int* m, double* Chat)
 // Copula distribution of the independence copula
 ////////////////////////////
 
-void C_ind(double* data, int* n, int* d, double* C)	
+void C_ind(double* data, int* n, int* d, double* C)
 {
 	int t=0, i=0;
-	
+
 	for(t=0;t<*n;t++)
 	{
 		for(i=0;i<*d;i++)
@@ -654,7 +655,7 @@ void C_ind(double* data, int* n, int* d, double* C)
 			else
 				C[t]=C[t] * data[t+1+(*n*i)-1];
 		}
-		
+
 	}
 }
 
@@ -665,7 +666,7 @@ void C_ind(double* data, int* n, int* d, double* C)
 // rest see above
 //////////////////////
 
-void gofECP2(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect, int* conindirect, double* par, double* par2, double* data, 
+void gofECP2(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect, int* conindirect, double* par, double* par2, double* data,
 		double* vv, double* vv2, int* calcupdate, double* statistic, int* statisticName)
 {
 	double *udata, *Chat1, *Chat2;
@@ -673,7 +674,7 @@ void gofECP2(int* T, int* d, int* family, int* maxmat, int* matrix, int* condire
 	udata = malloc(*d*(*T)*sizeof(double));
 	Chat1 = malloc(*T*sizeof(double));
 	Chat2 = malloc(*T*sizeof(double));
-	
+
 	for(t=0;t<*T;t++)
 	{
 		for(i=0;i<*d;i++)
@@ -689,9 +690,9 @@ void gofECP2(int* T, int* d, int* family, int* maxmat, int* matrix, int* condire
 
 	RvinePIT(T, d, family, maxmat, matrix, condirect, conindirect, par, par2, data, udata, vv, vv2, calcupdate);
 	ChatZj(udata, udata, T, d, T, Chat1);
-	
+
 	C_ind(udata,T,d,Chat2);
-	
+
 	*statistic=0;
 	if(*statisticName==3)	//Cramer-von Mises test statistic
 	{
@@ -708,7 +709,7 @@ void gofECP2(int* T, int* d, int* family, int* maxmat, int* matrix, int* condire
 		}
 		*statistic=*statistic*sqrt(*T);
 	}
-	
+
 	free(udata);
 	free(Chat1);
 	free(Chat2);
@@ -719,13 +720,13 @@ void gofECP2(int* T, int* d, int* family, int* maxmat, int* matrix, int* condire
 // p-value for ECP2
 ///////////////////////////
 
-void gofECP2_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect, int* conindirect, double* par, double* par2, double* data, 
+void gofECP2_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int* condirect, int* conindirect, double* par, double* par2, double* data,
 		double* vv, double* vv2, int* calcupdate, double* statistic, double* pvalue, int* statisticName, int* B)
 {
 	int i=0, j=0, m=0, t=0, *f;
 	double *bdata, bstat=0;
 	double *bvv, *bvv2;
-	
+
 	f = malloc(*T*sizeof(int));
 	bdata = malloc(*d*(*T)*sizeof(double));
 	bvv = malloc(*d*(*d)*(*T)*sizeof(double));
@@ -753,7 +754,7 @@ void gofECP2_pvalue(int* T, int* d, int* family, int* maxmat, int* matrix, int*
 		if(bstat>=*statistic)
 			*pvalue+=1.0/(*B);
 	}
-	
+
 	free(f);
 	free(bdata);
 	free(bvv);
